import os
import sys
import pathlib
import subprocess

__author__ = 'retif (https://github.com/retifrav), Borja González Seoane'
__copyright__ = 'Copyright 2019, retif (https://github.com/retifrav)'
__credits__ = 'retif (https://github.com/retifrav), Borja González Seoane'
__license__ = 'MIT'
# Source: https://github.com/retifrav/python-scripts/blob/master/generate-iconset/generate-iconset.py


def main(argv):
    if len(argv) < 1:
        print("No path to original / hi-res icon provided")
        raise SystemExit
    
    if len(argv) > 1:
        print("Too many arguments")
        raise SystemExit
    
    originalPicture = argv[0]
    if not (os.path.isfile(originalPicture)):
        print(f"There is no such file: {argv[0]}")
        raise SystemExit
    
    fname = pathlib.Path(originalPicture).stem
    ext = pathlib.Path(originalPicture).suffix
    destDir = pathlib.Path(originalPicture).parent
    
    iconsetDir = os.path.join(destDir, f"{fname}.iconset")
    if not (os.path.exists(iconsetDir)):
        pathlib.Path(iconsetDir).mkdir(parents=False, exist_ok=True)
    
    class IconParameters():
        width = 0
        scale = 1
        def __init__(self,width,scale):
            self.width = width
            self.scale = scale
        def getIconName(self):
            if self.scale != 1:
                return f"icon_{self.width}x{self.width}{ext}"
            else:
                return f"icon_{self.width//2}x{self.width//2}@2x{ext}"
    
    # https://developer.apple.com/design/human-interface-guidelines/macos/icons-and-images/app-icon#app-icon-sizes
    ListOfIconParameters = [
        IconParameters(16, 1),
        IconParameters(16, 2),
        IconParameters(32, 1),
        IconParameters(32, 2),
        IconParameters(64, 1),
        IconParameters(64, 2),
        IconParameters(128, 1),
        IconParameters(128, 2),
        IconParameters(256, 1),
        IconParameters(256, 2),
        IconParameters(512, 1),
        IconParameters(512, 2),
        IconParameters(1024, 1),
        IconParameters(1024, 2)
    ]
    
    # generate iconset
    for ip in ListOfIconParameters:
        subprocess.call(["sips", "-z", str(ip.width), str(ip.width), originalPicture, "--out", os.path.join(iconsetDir, ip.getIconName())])
        #print(f"Generated {ip.getIconName()}")
    
    # convert iconset to icns file
    subprocess.call(["iconutil", "-c", "icns", iconsetDir, "-o", os.path.join(destDir, f"{fname}.icns")])


if __name__ == "__main__":
    main(sys.argv)



# -*- coding: utf-8 -*-


"""folstamp main

This program takes an image that has well differentiated light and dark
tones and use it to create a custom stamped macOS folder icon.

This is the main Python file.
"""

import os
import sys
from copy import deepcopy
from typing import Tuple, List

from PIL import Image
from monoshape.__main__ import extract_shape

from folstamp.scripts import gen_iconset

__author__ = 'Borja González Seoane'
__copyright__ = 'Copyright 2019, Borja González Seoane'
__credits__ = 'Borja González Seoane'
__license__ = 'LICENSE'
__version__ = '1.1'
__maintainer__ = 'Borja González Seoane'
__email__ = 'dev@glezseoane.com'
__status__ = 'Development'


# =============================================
# =               Order handling              =
# =============================================
# noinspection PyShadowingNames
def handle_arguments(argv: List) -> Tuple:
    """Handles the arguments.

    Handles the arguments when the program is run and exports it into a
    enumeration of it.

    :param argv: Vector with the ordered input arguments
    :type argv: List
    :return: Interpreted argument vector
    :rtype: Tuple
    """
    # The input path is always the first argument and it is mandatory
    mask_path = argv.pop(0)
    if not mask_path.endswith('.png'):
        raise ValueError("The mask file has to be a PNG file and has to "
                         "end with the .png extension.")
    elif not os.path.isfile(mask_path):
        raise FileNotFoundError(mask_path)

    # The output path is always the last argument and it is mandatory
    outpath = argv.pop(len(argv) - 1)
    if not outpath.endswith('.png'):
        raise ValueError("The output file has to be a PNG file and has to "
                         "end with the .png extension.")
    # elif os.path.isfile(outpath):
    #     raise FileExistsError

    # Optional black background switch
    if '-bb' in argv:
        black_background = True
        argv.remove('-bb')
    elif '--black_background' in argv:
        black_background = True
        argv.remove('--black_background')
    else:
        black_background = False

    # Optional rgb switch
    if '-rgb' in argv:
        index = argv.index('-rgb')
        red = int(argv.pop(index + 1))
        green = int(argv.pop(index + 1))
        blue = int(argv.pop(index + 1))
        argv.remove('-rgb')
    else:
        # Uses default Mojave blue to the folder stamps
        red = 90
        green = 183
        blue = 227

    # If remains some param, it should be the optional template path argument
    if len(argv) == 1:
        template_path = argv[len(argv) - 2]
        if not template_path.endswith('.png'):
            raise ValueError("The template file has to be a PNG file and has "
                             "to end with the .png extension.")
        elif not os.path.isfile(template_path):
            raise FileNotFoundError
    elif not argv:  # Uses default Mojave folder icon
        template_path = str(os.path.dirname(
            os.path.abspath(__file__)).rsplit('/', 1)[0]) \
                        + '/util/GenericFolderIcon.png'
    else:
        raise IOError("Bad command composition! Please, read the manual.")

    return mask_path, black_background, template_path, outpath, \
        red, green, blue


# =============================================
# =                  Process                  =
# =============================================
def stamp(mask: 'Image.Image', background_folder: 'Image.Image') \
        -> 'Image.Image':
    """Stamps a mask on a background folder template

    :param mask: Mask to folstamp
    :type mask: 'Image.Image'
    :param background_folder: Background folder template
    :type background_folder: 'Image.Image'
    :return: Stamped folder image
    :rtype: 'Image.Image'
    """
    mask = deepcopy(mask)
    mask = mask.resize((500, 500))

    stamped_folder = deepcopy(background_folder)
    stamped_folder = stamped_folder.resize((1024, 1024))
    stamped_folder.paste(mask, (262, 300), mask)
    return stamped_folder


# =============================================
# =                    MAIN                   =
# =============================================
def main():
    mask_path, black_background, template_path, outpath, red, green, blue \
        = handle_arguments(sys.argv[1:])

    # Use Monoshape library to get the well format mask
    mask = extract_shape(mask_path, black_background, False,
                         True, red, green, blue)

    # Open the folder template to folstamp
    template_path = Image.open(template_path)
    template_path = template_path.convert("RGBA")

    # Stamp and save PNG
    stamped_folder = stamp(mask, template_path)
    stamped_folder.save(outpath, 'PNG')

    print(__file__)
    # Generate macOS iconset
    gen_iconset.main([outpath])

    print("[\033[92mOK\033[0m]: all generated!")


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-


"""folstamp setup module"""

from setuptools import setup

__author__ = 'Borja González Seoane'
__copyright__ = 'Copyright 2019, Borja González Seoane'
__credits__ = 'Borja González Seoane'
__license__ = 'LICENSE'
__version__ = '1.1'
__maintainer__ = 'Borja González Seoane'
__email__ = 'dev@glezseoane.com'
__status__ = 'Development'

setup(
    name='folstamp',
    version='1.1',
    packages=['folstamp', 'folstamp.scripts'],
    entry_points={
        'console_scripts': [
            'folstamp=folstamp.__main__:main',
        ],
    },
    python_requires='>=3.6',
    install_requires=['monoshape==1.1', 'pillow==6.1'],
    data_files=[('util', ['util/GenericFolderIcon.png', 'util/apple.png',
                          'util/green_folder.png']),
                ('share/man/man1', ['manpages/folstamp.1']),
                ("", ["LICENSE"])],
    url='https://github.com/GlezSeoane/folder-stamp',
    download_url='https://github.com/GlezSeoane/folder-stamp/archive/v1.0.1'
                 '.tar.gz',
    license='LICENSE',
    author='Borja González Seoane',
    author_email='dev@glezseoane.com',
    description='Generate custom macOS folder icons.',
    long_description='This program takes an image that has well '
                     'differentiated light and dark tones and use it to '
                     'create a custom stamped macOS folder icon.',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Operating System :: MacOS',
        'Topic :: Utilities',
        'Topic :: Artistic Software',
        'Topic :: Multimedia',
        'Topic :: Multimedia :: Graphics',
        'Topic :: Multimedia :: Graphics :: Editors',
        'Topic :: Multimedia :: Graphics :: Graphics Conversion',
    ],
)



# -*- coding: utf-8 -*-


"""folstamp scripts package

External scripts used in the performance of the program.
"""

__author__ = 'Borja González Seoane'
__copyright__ = 'Copyright 2019, Borja González Seoane'
__credits__ = 'Borja González Seoane, retif (https://github.com/retifrav)'
__license__ = 'LICENSE'
__version__ = '1.1'
__maintainer__ = 'Borja González Seoane'
__email__ = 'dev@glezseoane.com'
__status__ = 'Development'
